/**
 * @preserve
 * Copyright 2015-2016 Igor Bezkrovnyi
 * All rights reserved. (MIT Licensed)
 *
 * manhattanNeuQuant.ts - part of Image Quantization Library
 */
import { IDistanceCalculator } from "./common";
import { Point } from "../utils/point";
/**
 * Manhattan distance (NeuQuant modification) - w/o sRGB coefficients
 */
export declare class Manhattan implements IDistanceCalculator {
    protected _Pr: number;
    protected _Pg: number;
    protected _Pb: number;
    protected _Pa: number;
    protected _maxManhattanDistance: number;
    constructor();
    setWhitePoint(r: number, g: number, b: number, a: number): void;
    calculateNormalized(colorA: Point, colorB: Point): number;
    calculateRaw(r1: number, g1: number, b1: number, a1: number, r2: number, g2: number, b2: number, a2: number): number;
    protected _setDefaults(): void;
}
