export declare enum Direction {
    NONE = 0,
    UP = 1,
    LEFT = 2,
    RIGHT = 3,
    DOWN = 4,
}
export declare class HilbertCurveBase {
    private _x;
    private _y;
    private _d;
    private _width;
    private _height;
    private _callback;
    private _level;
    walk(width: number, height: number, visitorCallback: (x: number, y: number, d: number) => void): void;
    _walkHilbert(direction: Direction): void;
    private _visit(direction);
}
