/**
 * @preserve
 * Copyright 2015-2016 Igor Bezkrovnyi
 * All rights reserved. (MIT Licensed)
 *
 * manhattanSRGB.ts - part of Image Quantization Library
 */
import { IDistanceCalculator } from "./common";
import { Manhattan } from "./manhattan";
/**
 * Manhattan distance
 */
export declare class ManhattanSRGB extends Manhattan implements IDistanceCalculator {
    protected _setDefaults(): void;
}
