/**
 * @preserve
 * Copyright 2015-2016 Igor Bezkrovnyi
 * All rights reserved. (MIT Licensed)
 *
 * cmetric.ts - part of Image Quantization Library
 */
import { IDistanceCalculator } from "./common";
import { Point } from "../utils/point";
/**
 * TODO: Name it: http://www.compuphase.com/cmetric.htm
 */
export declare class CMETRIC implements IDistanceCalculator {
    private _rCoefficient;
    private _gCoefficient;
    private _bCoefficient;
    private _aCoefficient;
    private _maxDistance;
    constructor();
    setWhitePoint(r: number, g: number, b: number, a: number): void;
    calculateRaw(r1: number, g1: number, b1: number, a1: number, r2: number, g2: number, b2: number, a2: number): number;
    calculateNormalized(colorA: Point, colorB: Point): number;
}
