export declare class HueStatistics {
    private _numGroups;
    private _minCols;
    private _stats;
    private _groupsFull;
    constructor(numGroups: number, minCols: number);
    check(i32: number): void;
    injectIntoDictionary(histG: {
        [key: string]: number;
    }): void;
    injectIntoArray(histG: number[]): void;
}
