/**
 * @preserve
 * Copyright 2015-2016 Igor Bezkrovnyi
 * All rights reserved. (MIT Licensed)
 *
 * cie94.ts - part of Image Quantization Library
 */
import { IDistanceCalculator } from "./common";
import { Point } from "../utils/point";
/**
 * Computes CIE94 distance between 2 colors in LAB space.
 *
 * p1 = [l1, a1, b1]
 * p2 = [l2, a2, b2]
 * Returns distance:float
 *
 * Usage example:
 *     var d = CIE94_dist([94.0, -0.1, -0.55], [77.0, 0.5, 0.45])
 *
 * Iulius Curt, april 2013
 */
export declare class CIE94 implements IDistanceCalculator {
    private static _Kl;
    private static _K1;
    private static _K2;
    private _whitePoint;
    private _maxCIE94Distance;
    constructor();
    setWhitePoint(r: number, g: number, b: number, a: number): void;
    calculateRaw(r1: number, g1: number, b1: number, a1: number, r2: number, g2: number, b2: number, a2: number): number;
    calculateNormalized(colorA: Point, colorB: Point): number;
}
