/**
 * @preserve
 * Copyright 2015-2016 Igor Bezkrovnyi
 * All rights reserved. (MIT Licensed)
 *
 * ciede2000_original.ts - part of Image Quantization Library
 */
import { IDistanceCalculator } from "./common";
import { Point } from "../utils/point";
/**
 * CIEDE2000 algorithm (Original)
 */
export declare class CIEDE2000_Original implements IDistanceCalculator {
    private static _kL;
    private static _kC;
    private static _kH;
    private static _pow25to7;
    private static _deg360InRad;
    private static _deg180InRad;
    private static _deg30InRad;
    private static _deg6InRad;
    private static _deg63InRad;
    private static _deg275InRad;
    private static _deg25InRad;
    private _whitePoint;
    private _maxDistance;
    constructor();
    setWhitePoint(r: number, g: number, b: number, a: number): void;
    calculateRaw(r1: number, g1: number, b1: number, a1: number, r2: number, g2: number, b2: number, a2: number): number;
    calculateRawInLab(Lab1: {
        L: number;
        a: number;
        b: number;
    }, Lab2: {
        L: number;
        a: number;
        b: number;
    }): number;
    calculateNormalized(colorA: Point, colorB: Point): number;
    private _hp_f(b, aPrime);
    private _a_hp_f(C1, C2, h1p, h2p);
    private _dhp_f(C1, C2, h1p, h2p);
}
