/**
 * @preserve
 * Copyright 2015-2016 Igor Bezkrovnyi
 * All rights reserved. (MIT Licensed)
 *
 * euclideanRgbQuantWOAlpha.ts - part of Image Quantization Library
 */
import { IDistanceCalculator } from "./common";
import { Euclidean } from "./euclidean";
/**
 * Euclidean color distance (RgbQuant modification w/o Alpha)
 */
export declare class EuclideanRgbQuantWOAlpha extends Euclidean implements IDistanceCalculator {
    protected _setDefaults(): void;
}
