/**
 * @preserve TypeScript port:
 * Copyright 2015-2016 Igor Bezkrovnyi
 * All rights reserved. (MIT Licensed)
 *
 * rgbquant.ts - part of Image Quantization Library
 */
import { Palette } from "../../utils/palette";
import { PointContainer } from "../../utils/pointContainer";
import { IDistanceCalculator } from "../../distance/common";
import { IPaletteQuantizer } from "../common";
export declare class RGBQuant implements IPaletteQuantizer {
    private _colors;
    private _initialDistance;
    private _distanceIncrement;
    private _histogram;
    private _distance;
    constructor(colorDistanceCalculator: IDistanceCalculator, colors?: number, method?: number);
    sample(image: PointContainer): void;
    quantize(): Palette;
    private _buildPalette(idxi32);
}
