/**
 * @preserve
 * Copyright 2015-2016 Igor Bezkrovnyi
 * All rights reserved. (MIT Licensed)
 *
 * common.ts - part of Image Quantization Library
 */
import { PointContainer } from "../utils/pointContainer";
import { Palette } from "../utils/palette";
export interface IImageDitherer {
    quantize(pointBuffer: PointContainer, palette: Palette): PointContainer;
}
